import 'package:flutter/cupertino.dart';
import 'package:easyflu/base/request_info.dart';
import 'package:easyflu/config/const.dart';
import 'package:easyflu/http/exception/http_error.dart';
import 'package:easyflu/http/http_data.dart';
import 'package:easyflu/utils/object_util.dart';
import 'base_view_state.dart';

enum LoadType {
  Refresh,
  LoadMore,
}

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum = 0;

  int get currentPageNum => _currentPageNum;


  int? _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }

  bool? _hasMore;

  set hasMore(bool hasMore) => _hasMore = hasMore;

  bool get hasMore{
   if(_hasMore != null){
     return _hasMore!;
   }else{
     RequestInfo? requestInfo = getRequestInfo();
     return requestInfo?.useLoadMore ?? false? totalNum > list.length? true : false : false;
   }
  }



  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  Future<HttpData> getData(LoadType loadType) async{

    HttpData httpData = await requestList<List<T>, T>((httpData, requestInfo) {

      if(currentPageNum == firstPage()){
        list.clear();
      }
      list.addAll(httpData.data);
      onCompleted(list);
    }, isSetError: currentPageNum == firstPage());

    if(ObjectUtil.isEmpty(httpData.httpError)){
      setIdle();
    }

    bool success = true;

    if(ObjectUtil.isNotEmpty(httpData.httpError)){
      if(loadType == LoadType.LoadMore){
        _currentPageNum = --_currentPageNum;
      }

      if(httpData.httpError != null){
        if(httpData.httpError!.httpErrorType != HttpErrorType.EmptyDataError){
          success = false;
        }
      }

    }

    refreshController.finishRefresh(success: success);
    refreshController.finishLoad(success: success, noMore: !hasMore);

    return httpData;
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      getData(LoadType.Refresh);
    }
  }


  onCompleted(List<T> data) {}


  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  Future<HttpData> refreshData() {
    _currentPageNum = firstPage();
    return getData(LoadType.Refresh);
  }


  /// 上拉加载更多
  Future<HttpData> loadMore() {
    _currentPageNum = ++_currentPageNum;
    return getData(LoadType.LoadMore);
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  Widget buildEasyRefreshWidget({
    ScrollController? scrollController,
    Widget? footWidget,
    ViewStateWidgetBuilder? emptyWidgetBuilder,
    ViewStateWidgetBuilder? errorWidgetBuilder,
    ViewStateModelWidgetListBuilder<List<T>>? widgetListBuilder,
    ViewStateModelWidgetBuilder<List<T>>? widgetBuilder,
    Widget? busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function? onLoad,
    Function? onRefresh,
    }){
      return buildViewStateEasyRefresh<List<T>>(
        data: list,
        scrollController: scrollController,
        footWidget: footWidget,
        errorWidgetBuilder: errorWidgetBuilder,
        emptyWidgetBuilder: emptyWidgetBuilder,
        widgetListBuilder: widgetListBuilder,
        widgetBuilder: widgetBuilder,
        busyWidget: busyWidget,
        showBusyWidget: showBusyWidget,
        showEmptyWidget: showEmptyWidget,
        showErrorWidget: showErrorWidget,
        onLoad: hasMore? onLoad??(){
          loadMore();
        } : null,
        onRefresh: onRefresh??(){
          refreshData();
        }
      );
  }




}
