import 'package:extended_image/extended_image.dart';
import 'package:extended_list/extended_list.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:easyflu/base/base_view_state.dart';
import 'package:easyflu/config/const.dart';
import 'package:easyflu/easyflu.dart';
import 'package:easyflu/http/exception/http_error.dart';
import 'package:easyflu/utils/win_media.dart';


Widget buildNoDataWidget({BaseViewState? viewStateModel, Widget? child, GestureTapCallback? onTap}) => Container(
  height: double.infinity,
  alignment: Alignment.center,
  margin: getEdgeInsetsSymmetric(horizontal: defMargin),
  child: child ?? GestureDetector(
    onTap: onTap ?? (){
      viewStateModel?.requestWhenError();
    },
    child: Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [

        Spacer(),
        SizedBox(
            width: getWidth(100),
            height: getHeight(100),
            child: ExtendedImage(image: AssetImage('assets/images/nodata.png', package: 'flutter_mylibrary'))
        ),
        Text(
          viewStateModel?.httpError?.message ?? UNKNOWN_MSG,
          style: TextStyle(fontSize: getSp(16), color: Colors.grey[400]),
        ),
        Spacer(),
      ],
    ),
  ),
);


Widget buildViewStateWidget(
    BaseViewState viewState,
    Function idleWidget, {ViewStateWidgetBuilder? emptyWidgetBuilder,
      Widget? busyWidget,
      ViewStateWidgetBuilder? errorWidgetBuilder,
      Widget? errorDataWidget,
      bool showErrorWidget = true,
      bool showEmptyWidget = true,
      bool showBusyWidget = true
}){

  if(viewState.isIdle){
    return idleWidget();
  }else if(viewState.isError && viewState.httpError?.httpErrorType == HttpErrorType.EmptyDataError){
    if (showEmptyWidget) {
      if(emptyWidgetBuilder != null){
        return emptyWidgetBuilder.call(viewState);
      }else{
        return emptyWidget.call(viewState);
      }
    } else{
      return idleWidget();
    }
  }else if(viewState.isError){
    if (showErrorWidget) {
      if(errorDataWidget != null){
        return errorWidgetBuilder!.call(viewState);
      }else {
        return errorWidget.call(viewState);
      }
    } else{
      return idleWidget();
    }
  }else if (viewState.isBusy) {
    if (showBusyWidget) {
      return busyWidget??pageLoading;
    }else{
      return idleWidget();
    }
  } else{
    return Container();
  }
}

Widget? buildRefreshNoDataWidget({BaseViewState? viewState,
      Widget? busyWidget,
      ViewStateWidgetBuilder? emptyWidgetBuilder,
      ViewStateWidgetBuilder? errorWidgetBuilder,
      bool showErrorWidget = true,
      bool showEmptyWidget = true,
      bool showBusyWidget = true
}){
  if(viewState != null){
    if(viewState.isError){
      if(viewState.httpError?.httpErrorType == HttpErrorType.EmptyDataError && showEmptyWidget){
        if(emptyWidgetBuilder != null){
          return emptyWidgetBuilder.call(viewState);
        }else {
          return emptyWidget.call(viewState);
        }
      }else{
        if(showErrorWidget){
          if(emptyWidgetBuilder != null){
            return errorWidgetBuilder!.call(viewState);
          }else {
            return errorWidget.call(viewState);
          }
        }else{
          return null;
        }
      }
    }else if (viewState.isBusy && showBusyWidget) {
      return busyWidget??pageLoading;
    }else{
      return null;
    }
  }else{
    return null;
  }

}






/// 间隔
class Gaps {
  /// 水平间隔
  static Widget hGap5 = new SizedBox(width: getWidth(5));
  static Widget hGap10 = new SizedBox(width: getWidth(10));
  static Widget hGap15 = new SizedBox(width: getWidth(15));
  static Widget hGapDef = new SizedBox(width: getWidth(defMargin));

  /// 垂直间隔
  static Widget vGap5 = new SizedBox(height: getHeight(5));
  static Widget vGap10 = new SizedBox(height: getHeight(10));
  static Widget vGap15 = new SizedBox(height: getHeight(15));
  static Widget vGapDef = new SizedBox(width: getHeight(defMargin));


  /// 自定义垂直间隔
  static Widget getVertical(double height){
    return SizedBox(
      height: getHeight(height),
    );
  }

  /// 自定义水平间隔
  static Widget getHorizontal(double width){
    return SizedBox(
      width: getHeight(width),
    );
  }
}



/// ******************************公共控件******************************/
PreferredSizeWidget buildAppBarWidget({
  List<Widget>? actions,
  Color? backgroundColor,
  Widget? leading,
  bool? automaticallyImplyLeading,
  TextStyle? textStyle,
  Widget? titleWidget,
  FontWeight? fontWeight,
  bool centerTitle = true,
  String title = '',
  double? titleSpacing,
  PreferredSizeWidget? bottom,
  double? leadingWidth,
  double? toolbarHeight,
}){
  Widget mTitleWidget;

  if(fontWeight != null || textStyle != null){
    mTitleWidget = buildText(title, fontWeight: fontWeight, textStyle: textStyle);
  }else{
    mTitleWidget = Text(title);
  }


  return AppBar(
    toolbarHeight: toolbarHeight,
    leadingWidth: leadingWidth,
    automaticallyImplyLeading: automaticallyImplyLeading??centerTitle,
    titleSpacing: titleSpacing,
    title: titleWidget??mTitleWidget,
    bottom: bottom,
    leading: leading,
    centerTitle: centerTitle,
    backgroundColor: backgroundColor,
    actions: actions,
  );
}



///获取SafeArea的布局
Widget buildScaffoldWidget({
  required Widget body,
  double? mToolBarHeight,
  Color? backgroundColor,
  Color? appBarBackColor,
  Widget? leading,
  bool useAppBar = true,
  bool automaticallyImplyLeading = true,
  TextStyle? textStyle,
  Widget? titleWidget,
  double? hMargin,
  double? titleTextSize,
  FontWeight? fontWeight,
  bool centerTitle = true,
  String title = '',
  double? titleSpacing,
  EdgeInsets? padding,
  Widget? bottomNavigationBar,
  double? leadingWidth,
  PreferredSizeWidget? bottom,
  bool useSafeArea = true,
  List<Widget>? actions,
  bool resizeToAvoidBottomInset = false,
  PreferredSize? appBar,
}){
  double mAppBarHeight = mToolBarHeight??appBarHeight;
  return Scaffold(
    backgroundColor: backgroundColor,
    appBar: useAppBar? appBar??buildAppBarWidget(
        toolbarHeight: mAppBarHeight,
        leadingWidth: leadingWidth,
        title: title,
        automaticallyImplyLeading: automaticallyImplyLeading,
        titleWidget: titleWidget,
        fontWeight: fontWeight,
        leading: leading,
        titleSpacing: titleSpacing,
        centerTitle: centerTitle,
        textStyle: textStyle,
        backgroundColor: appBarBackColor,
        bottom: bottom,
        actions: actions
    ):  null,
    resizeToAvoidBottomInset: resizeToAvoidBottomInset,
    bottomNavigationBar: bottomNavigationBar,
    body: useSafeArea? SafeArea(
      child: Padding(
        padding: padding??getEdgeInsetsAll(0),
        child: body,
      ),
    ) : Padding(
      padding: padding??getEdgeInsetsAll(0),
      child: body,
    ),
  );
}



Widget buildText(String text, {TextStyle? textStyle,
      int? maxLines,
      double fontSize = 14,
      Color? color,
      EdgeInsets? margin,
      TextDecoration? textDecoration,
      TextAlign? textAlign,
      FontWeight? fontWeight,
      TextOverflow? overflow
}){
  return Container(
    margin: margin,
    child: Text(
      text,
      maxLines: maxLines,
      overflow: overflow,
      textAlign: textAlign,
      style: textStyle??TextStyle(
          decoration: textDecoration,
          fontSize: getSp(fontSize),
          fontWeight: fontWeight,
          color: color
      ),
    ),
  );
}







///间隔listview
Widget buildListViewSeparated({
  bool closeToTrailing = false,
  bool useKeFrame = true,
  ViewPortBuilder? viewportBuilder,
  required int itemCount,
  required IndexedWidgetBuilder itemBuilder,
  Axis scrollDirection = Axis.vertical,
  double dividerHeight = 15,
  Color dividerColor = Colors.transparent,
  ScrollPhysics? physics,
  EdgeInsets parentPadding = EdgeInsets.zero,
  EdgeInsets itemPadding = EdgeInsets.zero,
  bool useSliver = false,
  bool shrinkWrap = false,
  bool useListView = true,
  CollectGarbage? collectGarbage,
  bool includeStart = true,
  bool includeEnd = true,
  Widget? placeHolder,
  int estimateCount = 0,
  ScrollController? controller,
}){

  int count = itemCount * 2 - 1;
  bool useEven = false;
  if(includeStart && includeEnd){
    count = itemCount * 2 + 1;
    useEven = true;
  }else if(includeStart){
    count = itemCount * 2;
    useEven = true;
  }else if(includeEnd){
    count = itemCount * 2;
    useEven = false;
  }else{
    count = itemCount * 2 - 1;
    useEven = false;
  }

  Widget child = Container();

  if (useSliver) {
    if (itemCount <= 0) {
      child = SliverToBoxAdapter(
        child: Container(
          width: 0,
          height: 0,
        ),
      );
    }
    if (useListView) {
      if (scrollDirection == Axis.vertical) {
        child = ExtendedSliverList(
          delegate: SliverChildBuilderDelegate((context, index){
            return _buildListViewItem(
                useEven: useEven,
                useKeFrame: useKeFrame,
                scrollDirection: scrollDirection,
                dividerHeight: dividerHeight,
                dividerColor: dividerColor,
                padding: itemPadding,
                context: context,
                index: index,
                itemBuilder: itemBuilder
            );}, childCount: count
          ),
          extendedListDelegate: ExtendedListDelegate(
            ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
              viewportBuilder: (int fistIndex, int lastIndex){
                if(viewportBuilder != null){
                  viewportBuilder(fistIndex, lastIndex);
                }
              },
              closeToTrailing: closeToTrailing,
              collectGarbage: collectGarbage,
          ),
        );
      }else{
        child = SliverToBoxAdapter(
          child: ExtendedListView.builder(
            controller: controller,
            shrinkWrap: shrinkWrap,
            physics: physics,
            itemCount: count,
            itemBuilder: (context, index){
              return _buildListViewItem(
                  useEven: useEven,
                  useKeFrame: useKeFrame,
                  scrollDirection: scrollDirection,
                  dividerHeight: dividerHeight,
                  dividerColor: dividerColor,
                  padding: itemPadding,
                  context: context,
                  index: index,
                  itemBuilder: itemBuilder
              );
            },
            scrollDirection: scrollDirection,
            extendedListDelegate: ExtendedListDelegate(
              ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
                viewportBuilder: (int fistIndex, int lastIndex){
                  if(viewportBuilder != null){
                    viewportBuilder(fistIndex, lastIndex);
                  }
                },
                closeToTrailing: closeToTrailing,
                collectGarbage: collectGarbage
            ),

          ),
        );
      }
    } else{
      List<Widget> widgetList = List.generate(count, (index){
        return _buildListViewItem(
            useEven: useEven,
            scrollDirection: scrollDirection,
            dividerHeight: dividerHeight,
            dividerColor: dividerColor,
            padding: itemPadding,
            context: context,
            index: index,
            itemBuilder: itemBuilder
        );
      });

      ///使用Column或者Row
      child = SliverToBoxAdapter(
        child: SingleChildScrollView(
          controller: controller,
          scrollDirection: scrollDirection,
          physics: physics,
          child: scrollDirection == Axis.vertical? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList,
          ) : Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList,
          ),
        ),
      );
    }
  }else{
    if (itemCount <= 0) {
      return Container(
        width: 0,
        height: 0,
      );
    }
    if (useListView) {
      child = ExtendedListView.builder(
          controller: controller,
          shrinkWrap: shrinkWrap,
          physics: physics,
          itemCount: count,
          scrollDirection: scrollDirection,
          extendedListDelegate: ExtendedListDelegate(
            ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
              viewportBuilder: (int fistIndex, int lastIndex){
                if(viewportBuilder != null){
                  viewportBuilder(fistIndex, lastIndex);
                }
              },
              closeToTrailing: closeToTrailing,
              collectGarbage: collectGarbage
          ),
          itemBuilder: (context, index){
            return _buildListViewItem(
                useEven: useEven,
                useKeFrame: useKeFrame,
                scrollDirection: scrollDirection,
                dividerHeight: dividerHeight,
                dividerColor: dividerColor,
                context: context,
                index: index,
                padding: itemPadding,
                itemBuilder: itemBuilder
            );
          }
      );
    }else{

      List<Widget> widgetList = List.generate(count, (index){
        return _buildListViewItem(
            useEven: useEven,
            context: context,
            useKeFrame: useKeFrame,
            index: index,
            scrollDirection: scrollDirection,
            dividerHeight: dividerHeight,
            dividerColor: dividerColor,
            padding: itemPadding,
            itemBuilder: itemBuilder
        );
      });
      ///使用Column或者Row
      child = SingleChildScrollView(
        controller: controller,
        scrollDirection: scrollDirection,
        physics: physics,
        child: scrollDirection == Axis.vertical? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList
        ) : Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList
        ),
      );
    }
  }

  if(useKeFrame){
    return SizeCacheWidget(
      estimateCount: estimateCount,
      child: Padding(
        padding: parentPadding,
        child: child,
      ),
    );
  }else{
    return Padding(
      padding: parentPadding,
      child: child,
    );
  }

}


Widget _buildListViewItem({
  bool? useEven,
  int? index,
  bool? useKeFrame,
  BuildContext? context,
  Axis? scrollDirection,
  double? dividerHeight,
  Color? dividerColor,
  EdgeInsets padding = EdgeInsets.zero,
  IndexedWidgetBuilder? itemBuilder,
  Widget? placeHolder,
}){
  Widget child = Container();
  if(useEven!? index!.isEven : index!.isOdd){
    child = Container(
      width: scrollDirection == Axis.horizontal ? getWidth(dividerHeight!) : null,
      height: scrollDirection == Axis.vertical ? getHeight(dividerHeight!) : null,
      color: dividerColor,
    );
  }else{
    child = itemBuilder!(context!, index ~/ 2);
  }


  return FrameSeparateWidget(
    placeHolder: placeHolder??Container(),
    index: index,
    child: Padding(
      padding: padding,
      child: child,
    ),
  );
}





///胶囊控件
Widget buildSmartContainer({
  bool visible = true,
  BoxConstraints? constraints,
  bool userClip = false,
  AlignmentGeometry? alignment,
  GestureTapCallback? onTap,
  bool isCircle = false,
  double borderWidth = 0,
  Color backgroundColor = Colors.white,
  double radius = 0,
  Widget? child,
  EdgeInsets padding = EdgeInsets.zero,
  Color borderColor = Colors.transparent,
  EdgeInsets? margin,
  BorderRadius? borderRadius,
  double? height,
  double? width,
  HitTestBehavior? behavior,
  Offset? offset,
  Color? boxColor,
  double blurRadius = 0,
  double spreadRadius = 0,
  List<BoxShadow>? boxShadow
}){
  return Visibility(
    visible: visible,
    child: GestureDetector(
        behavior: behavior,
        onTap: onTap,
        child: userClip? ClipRRect(
          borderRadius: borderRadius ?? BorderRadius.all(Radius.circular(getRadius(radius))),
          child: Container(
            constraints: constraints,
            color: backgroundColor,
            width: width,
            height: height,
            margin: margin,
            padding: padding,
            alignment: alignment,
            child: child??Container(),
          ),
        ): Container(
          constraints: constraints,
          width: width,
          height: height,
          margin: margin,
          padding: padding,
          alignment: alignment,
          decoration: BoxDecoration(
              shape: isCircle? BoxShape.circle : BoxShape.rectangle,
              color: backgroundColor,
              borderRadius: isCircle? null : borderRadius??BorderRadius.all(Radius.circular(radius)),
              border: borderWidth == 0? null : Border.all(color: borderColor, width: borderWidth),
              boxShadow: boxShadow
          ),
          child: child??Container(),
        )
    ),
  );
}




///阴影框
Widget buildBoxShadow({
  GestureTapCallback? onTap,
  AlignmentGeometry? alignment,
  double? height,
  double? width,
  Offset? offset,
  Color? boxColor,
  double blurRadius = 10,
  BorderRadiusGeometry? borderRadiusGeometry,
  Widget? child,
  EdgeInsetsGeometry? padding,
  BoxBorder? border,
  EdgeInsets? margin,
  double radius = 5,
  Color? backgroundColor,
}){
  return GestureDetector(
    onTap: onTap,
    child: Container(
        margin: margin,
        padding: padding,
        width: width,
        height: height,
        alignment: alignment,
        decoration: BoxDecoration(
            border: border,
            color: backgroundColor,
            boxShadow: [
              BoxShadow(
                offset: offset??Offset(5.0, 5.0),
                color: boxColor??Color(0xff9cb3b3b3),//阴影颜色
                blurRadius: blurRadius,//阴影大小
              )
            ],
            borderRadius: borderRadiusGeometry??BorderRadius.all(Radius.circular(radius))
        ),
        child: child
    ),
  );
}





///圆角图片
Widget buildConnerNetworkImage({LoadStateChanged? loadStateChanged,
  bool enableLoadState = false,
  bool cache = true,
  bool enableMemoryCache = true,
  bool clearMemoryCacheIfFailed = true,
  required String url,
  double width = double.infinity,
  double height = double.infinity,
  bool isCircle = false,
  double radius = 5,
  BoxFit fit = BoxFit.fill
}){
  return ExtendedImage(
    image: ExtendedNetworkImageProvider(
      url,
      cache: cache,
    ),
    width: getWidth(width),
    height: getHeight(height),
    shape: isCircle? BoxShape.circle : BoxShape.rectangle,
    borderRadius: BorderRadius.all(Radius.circular(radius)),
    enableMemoryCache: enableMemoryCache,
    clearMemoryCacheIfFailed: clearMemoryCacheIfFailed,
    enableLoadState: enableLoadState,
    loadStateChanged: loadStateChanged,
    fit: fit,
  );
}




